<?php

use App\Enums\UserType;
use App\Exceptions\InvalidOtpException;
use App\Models\User;
use App\Models\Otp;
use App\Notifications\VerifyMobileNumber;
use Illuminate\Database\Eloquent\Factories\Sequence;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Notification;

use function Pest\Laravel\actingAs;
use function Pest\Laravel\assertDatabaseCount;
use function Pest\Laravel\assertDatabaseHas;
use function PHPUnit\Framework\assertTrue;

test('otp can be created', function () {
    Notification::fake();
    $user = User::factory()->unverifiedNumber()->assignRole(UserType::User)->create();
    $response = actingAs($user)->post('api/send-verification-otp');
    Notification::assertSentTo(
        $user,
        VerifyMobileNumber::class,
        function (VerifyMobileNumber $verifyMobileNumber)  use ($user) {
            assertDatabaseHas('otps', [
                'user_id' => $user->id,
                'is_used' => false
            ]);
            assertDatabaseCount('otps', 1);
            return true;
        }
    );
    $response->assertStatus(201);
    $response->assertSee('Your OTP Has been seen to your phone number');

});

test('otp does not get created when the user has already been verified', function () {
    Notification::fake();
    $user = User::factory()->assignRole(UserType::User)->create();
    $response = actingAs($user)->post('api/send-verification-otp');
    Notification::assertNothingSentTo($user);
});

test('users can verify otp', function () {
    Notification::fake();
    $user = User::factory()->unverifiedNumber()->assignRole(UserType::User)->create();
    $response = actingAs($user)->post('/api/send-verification-otp');
    Notification::assertSentTo(
        $user,
        VerifyMobileNumber::class,
        function (VerifyMobileNumber $verifyMobileNumber)  use ($user) {
            $response = actingAs($user)->post('/api/verify-otp', [
                'code' => $verifyMobileNumber->otp
            ]);
            $response->assertSee('You have successfully verified you phone number');
            assertTrue($user->fresh()->hasVerifiedMobileNumber(), 'user is unable to verify mobile number');
            assertDatabaseHas('otps', [
                'user_id' => $user->id,
                'is_used' => true
            ]);
            return true;
        }
    );
});

test('users cannot verify mobile number with previous otp', function ($code) {
    $user = User::factory()
                ->unverifiedNumber()
                ->has(
                    Otp::factory()
                        ->count(3)
                        ->state(new Sequence(
                            ['code' => Hash::make('1234'), 'created_at' => now()->subMinutes(rand(0, 1000))], //randomize the created_at time because factory uses the same time
                            ['code' => Hash::make('3333'), 'created_at' => now()->subMinutes(rand(0, 1000))],
                            ['code' => Hash::make('5555')]
                        ))
                )
                ->create();
    $response = actingAs($user)
                    ->post('/api/verify-otp', [
                        'code' => $code
                    ]);
    $response
        ->assertStatus(400)
        ->assertSee('The code you have entered is invalid or has expired');

})
// ->throws(InvalidOtpException::class)
->with(['1234', '3333']);
